@@ -2,9 +2,6 @@ import pandas
 import logging
 import collections
 import numpy as np
-import matplotlib
-
-matplotlib.use("agg")
 from skfuzzy import control as ctrl
 from skfuzzy import gaussmf
 import sklearn
