@@ -4,6 +4,8 @@ import random
 import types
 import unittest
 
+import numpy.testing
+
 try:
     import unittest.mock as mock
 except ImportError:
@@ -45,7 +47,7 @@ def find_path_to_test_db(dbfn="TestDb.h5"):
         raise IOError("cannot access {}. (Tried {} and {})".format(dbfn, path1, path2))
 
 
-class DatabaseTests(unittest.TestCase):
+class TestWithDbInstance(unittest.TestCase):
     db = None
 
     @classmethod
@@ -56,8 +58,10 @@ class DatabaseTests(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        cls.db.get_hdf5_handle().close()
+        cls.db.close()
 
+
+class DatabaseTests(TestWithDbInstance):
     def test_get_vpairs_of_entry_with_orthologs(self):
         for entry_nr, exp_vps_cnt in [(12, 3), (1, 0), (4, 1)]:
             vps = self.db.get_vpairs(entry_nr)
@@ -68,6 +72,23 @@ class DatabaseTests(unittest.TestCase):
                 sorted(vps.dtype.fields.keys()),
             )
 
+    def test_get_vpairs_between_species_is_same_as_loading_all_vpairs(self):
+        g1, g2 = "YEAST", "SCHPO"
+        t0 = time.time()
+        rng1 = self.db.id_mapper["OMA"].genome_range(g1)
+        rng2 = self.db.id_mapper["OMA"].genome_range(g2)
+        expected = []
+        for en in range(rng1[0], rng1[1] + 1):
+            for vp in self.db.get_vpairs(en):
+                if rng2[0] <= vp["EntryNr2"] <= rng2[1]:
+                    expected.append(vp)
+        expected = numpy.fromiter(expected, dtype=expected[0].dtype)
+        t1 = time.time()
+        actual = self.db.get_vpairs_between_species_pair(g1, g2)
+        t2 = time.time()
+        numpy.testing.assert_equal(expected, actual)
+        self.assertLess(10 * (t2 - t1), t1 - t0, "expect at least 10x speedup")
+
     def test_neighborhood_close_to_boundary(self):
         query, window = 3, 7
         neighbors, idx = self.db.neighbour_genes(query, window)
@@ -390,6 +411,11 @@ class XRefIdMapperTest(unittest.TestCase):
         self.assertEqual((expected_len,), all_mapped.shape)
         self.assertEqual(self.xrefmapper.xref_tab.dtype, all_mapped.dtype)
 
+    def test_entry_nr_range(self):
+        all_mapped = self.xrefmapper.map_many_entry_nrs(numpy.arange(1, 4))
+        range_mapped = self.xrefmapper.map_entry_nr_range(1, 4)
+        numpy.testing.assert_equal(all_mapped, range_mapped)
+
     def test_map_entry_iterator(self):
         it = self.xrefmapper.iter_xrefs_for_entry_nr(1)
         self.assertTrue(isinstance(it, types.GeneratorType), "not a generator")
@@ -733,5 +759,29 @@ class FastMapperTester(unittest.TestCase):
         self.assertIn("Skipping", "\n".join(cm.output))
         self.assertEqual(0, len(anno))
 
-    def test_search_existing_sequance(self):
-        pass
+    def check_mapped_seqs(self, seqs, way="Exact"):
+        with self.assertLogs("pyoma", level=logging.INFO) as cm:
+            anno = list(self.fast_mapper.iter_projected_goannotations(seqs))
+        for a in anno:
+            with_ = self.fast_mapper.db.id_mapper["OMA"].map_entry_nr(
+                int(a["DB_Object_ID"])
+            )
+            self.assertEqual(f"{way}:{with_}".split(":"), a["With"].split(":")[:2])
+        self.assertGreaterEqual(len(a), 1)
+
+    def test_search_existing_sequences(self):
+        seqs = [
+            SeqRecord(
+                id=f"{enr}", seq=Seq(self.fast_mapper.db.get_sequence(enr).decode())
+            )
+            for enr in range(1, 5)
+        ]
+        self.check_mapped_seqs(seqs, way="Exact")
+
+    def test_search_inexact_search_seq(self):
+        seqs = []
+        for en in range(1, 5):
+            s = self.fast_mapper.db.get_sequence(en).decode()
+            modif_seq = s.replace("A", "R")
+            seqs.append(SeqRecord(id=f"{en}", seq=Seq(modif_seq)))
+        self.check_mapped_seqs(seqs, way="Approx")
